/*
 * Copyright 1991-1998, Brown University, Providence, RI.
 * 
 *                         All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Brown University not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
 * PARTICULAR PURPOSE.  IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR
 * ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/************************************************************************
*									*
*   iswap.h								*
*									*
************************************************************************/

#ifdef NEED_PROTO
typedef void (*iswap_func_t)(u8_t *, u8_t *, int, int, int, int, int);
#else
typedef void (*iswap_func_t)();
#endif

static unsigned char rbt[256] = {	/* reversed bits table */
	0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
	0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
	0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
	0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
	0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
	0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
	0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
	0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
	0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
	0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
	0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
	0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
	0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
	0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
	0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
	0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
	0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
	0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
	0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
	0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
	0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
	0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
	0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
	0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
	0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
	0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
	0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
	0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
	0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
	0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
	0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
	0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff};

static unsigned char rnt[256] = {	/* reversed nibbles table */
        0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70,
        0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf0,
        0x01, 0x11, 0x21, 0x31, 0x41, 0x51, 0x61, 0x71,
        0x81, 0x91, 0xa1, 0xb1, 0xc1, 0xd1, 0xe1, 0xf1,
        0x02, 0x12, 0x22, 0x32, 0x42, 0x52, 0x62, 0x72,
        0x82, 0x92, 0xa2, 0xb2, 0xc2, 0xd2, 0xe2, 0xf2,
        0x03, 0x13, 0x23, 0x33, 0x43, 0x53, 0x63, 0x73,
        0x83, 0x93, 0xa3, 0xb3, 0xc3, 0xd3, 0xe3, 0xf3,
        0x04, 0x14, 0x24, 0x34, 0x44, 0x54, 0x64, 0x74,
        0x84, 0x94, 0xa4, 0xb4, 0xc4, 0xd4, 0xe4, 0xf4,
        0x05, 0x15, 0x25, 0x35, 0x45, 0x55, 0x65, 0x75,
        0x85, 0x95, 0xa5, 0xb5, 0xc5, 0xd5, 0xe5, 0xf5,
        0x06, 0x16, 0x26, 0x36, 0x46, 0x56, 0x66, 0x76,
        0x86, 0x96, 0xa6, 0xb6, 0xc6, 0xd6, 0xe6, 0xf6,
        0x07, 0x17, 0x27, 0x37, 0x47, 0x57, 0x67, 0x77,
        0x87, 0x97, 0xa7, 0xb7, 0xc7, 0xd7, 0xe7, 0xf7,
        0x08, 0x18, 0x28, 0x38, 0x48, 0x58, 0x68, 0x78,
        0x88, 0x98, 0xa8, 0xb8, 0xc8, 0xd8, 0xe8, 0xf8,
        0x09, 0x19, 0x29, 0x39, 0x49, 0x59, 0x69, 0x79,
        0x89, 0x99, 0xa9, 0xb9, 0xc9, 0xd9, 0xe9, 0xf9,
        0x0a, 0x1a, 0x2a, 0x3a, 0x4a, 0x5a, 0x6a, 0x7a,
        0x8a, 0x9a, 0xaa, 0xba, 0xca, 0xda, 0xea, 0xfa,
        0x0b, 0x1b, 0x2b, 0x3b, 0x4b, 0x5b, 0x6b, 0x7b,
        0x8b, 0x9b, 0xab, 0xbb, 0xcb, 0xdb, 0xeb, 0xfb,
        0x0c, 0x1c, 0x2c, 0x3c, 0x4c, 0x5c, 0x6c, 0x7c,
        0x8c, 0x9c, 0xac, 0xbc, 0xcc, 0xdc, 0xec, 0xfc,
        0x0d, 0x1d, 0x2d, 0x3d, 0x4d, 0x5d, 0x6d, 0x7d,
        0x8d, 0x9d, 0xad, 0xbd, 0xcd, 0xdd, 0xed, 0xfd,
        0x0e, 0x1e, 0x2e, 0x3e, 0x4e, 0x5e, 0x6e, 0x7e,
        0x8e, 0x9e, 0xae, 0xbe, 0xce, 0xde, 0xee, 0xfe,
        0x0f, 0x1f, 0x2f, 0x3f, 0x4f, 0x5f, 0x6f, 0x7f,
        0x8f, 0x9f, 0xaf, 0xbf, 0xcf, 0xdf, 0xef, 0xff};
/*
**	The swap routines
*/
extern void iswap_cpy();
extern void iswap_nib();
extern void iswap___w();
extern void iswap__b_();
extern void iswap__b3();
extern void iswap__bw();
extern void iswap_r__();
extern void iswap_r_w();
extern void iswap_rb_();
extern void iswap_rbw();
/*
**	To make the table that follows pretty
*/
#define ___ iswap_cpy
#define nib iswap_nib
#define __w iswap___w
#define _b_ iswap__b_
#define _b3 iswap__b3
#define _bw iswap__bw
#define r__ iswap_r__
#define r_w iswap_r_w
#define rb_ iswap_rb_
#define rbw iswap_rbw
/*
**	Essentially the same table of functions that appears in the
**	MIT distribution in the XPutImage code.  I borrowed the
**	format notation from there.
**
**	indexed by (bit_order)+(byte_order*2)+(scan_unit/16)*4
**
**	{1,2,4} is the scanline-unit	r is reverse bits
**	{M,L} is the byte order		b is swap bytes in shorts
**	{m,l} is the bit order		w is swap shorts in words
*/
static iswap_func_t iswapXYfunc[12][12] = {
/*        1Ll  1Lm  1Ml  1Mm  2Ll  2Lm  2Ml  2Mm  4Ll  4Lm  4Ml  4Mm	*/
/* 1Ll */{___, r__, ___, r__, ___, rb_, _b_, r__, ___, rbw, _bw, r__ },
/* 1Lm */{r__, ___, r__, ___, r__, _b_, rb_, ___, r__, _bw, rbw, ___ },
/* 1Ml */{___, r__, ___, r__, ___, rb_, _b_, r__, ___, rbw, _bw, r__ },
/* 1Mm */{r__, ___, r__, ___, r__, _b_, rb_, ___, r__, _bw, rbw, ___ },
/* 2Ll */{___, r__, ___, r__, ___, rb_, _b_, r__, ___, rbw, _bw, r__ },
/* 2Lm */{rb_, _b_, rb_, _b_, rb_, ___, r__, _b_, rb_, __w, r_w, _b_ },
/* 2Ml */{_b_, rb_, _b_, rb_, _b_, r__, ___, rb_, _b_, r_w, __w, rb_ },
/* 2Mm */{r__, ___, r__, ___, r__, _b_, rb_, ___, r__, _bw, rbw, ___ },
/* 4Ll */{___, r__, ___, r__, ___, rb_, _b_, r__, ___, rbw, _bw, r__ },
/* 4Lm */{rbw, _bw, rbw, _bw, rbw, __w, r_w, _bw, rbw, ___, r__, _bw },
/* 4Ml */{_bw, rbw, _bw, rbw, _bw, r_w, __w, rbw, _bw, r__, ___, rbw },
/* 4Mm */{r__, ___, r__, ___, r__, _b_, rb_, ___, r__, _bw, rbw, ___ }
};
/*
**	Z format swap functions, indexed by bits_per_pixel/8
**
**	nib is swap nibbles
**	3b is swap 3 bytes in a 24-bit unit
*/
static iswap_func_t iswapZfunc[5] = { /*   0    1    2    3    4	*/
                                          nib, ___, _b_, _b3, _bw
};

#undef ___
#undef nib
#undef __w
#undef _b_
#undef _b3
#undef _bw
#undef r__
#undef r_w
#undef rb_
#undef rbw
